@@ -1,11 +1,12 @@
 import json
 from weakref import ref
 from typing import (Mapping, Sequence, List,
-                    Callable, Optional, Union)
+                    Callable, Optional, Union,
+                    Tuple)
 from collections import abc
 
 
-__version__ = '0.6.0'
+__version__ = '0.8.0'
 
 __all__ = [
     'SchemaError',
@@ -171,33 +172,54 @@ class Scheme(abc.Mapping):
         super().__init_subclass__(**kwargs)
         _fields = set()
         _aliases = {}
-        for name, value in cls.__dict__.items():
-            if isinstance(value, Field):
-                _fields.add(name)
-                if value.alias is not None:
-                    _aliases[value.alias] = name
+
+        # Get only parents without std base classes
+        base_class_count = len(Scheme.mro())
+        pure_mro = cls.mro()[:-base_class_count]
+
+        for base in reversed(pure_mro):
+            if not issubclass(base, Scheme):
+                continue
+            for name, value in base.__dict__.items():
+                # if attr was overriden in class dont inherit it
+                if base != cls and name in cls.__dict__:
+                    continue
+                if isinstance(value, Field):
+                    _fields.add(name)
+                    if value.alias is not None:
+                        _aliases[value.alias] = name
+
         cls._fields = frozenset(_fields)
         cls._aliases = _aliases
         cls._root = root
+
         _schemes_map[cls.__name__] = cls
 
+    @classmethod
+    def get_best_match(cls, keys: set) -> Tuple[int, Optional['Scheme']]:
+        if not cls._root:
+            return (-1, cls)
+
+        best_match = (0, None)
+        for i in cls.__subclasses__():
+
+            if i._root:
+                coverage, i = i.get_best_match(keys)
+            else:
+                coverage = i.coverage(keys)
+
+            if coverage > best_match[0]:
+                best_match = (coverage, i)
+
+        return best_match
+
     @classmethod
     def from_dict(cls, d: Mapping) -> 'Scheme':
         assert(isinstance(d, Mapping))
 
-        coverage = None
+        coverage, cls = cls.get_best_match(set(d.keys()))
 
-        if cls._root:
-            struct_map = {}
-            for i in cls.__subclasses__():
-                struct_map[i.coverage(set(d.keys()))] = i
-            coverage = max(struct_map.keys())
-            cls = struct_map[coverage]
-
-        if coverage is None:
-            coverage = cls.coverage(set(d.keys()))
-
-        if not coverage:
+        if cls is None:
             raise SchemaError(
                 'Cannot map structure none of the fields match!\n'
                 f'Scheme: {cls.__qualname__}\n'
@@ -268,7 +290,7 @@ class Scheme(abc.Mapping):
             if value is not None or leave_none:
                 key = aliases.get(key, key)
                 if isinstance(value, Scheme):
-                    value = value.as_dict()
+                    value = value.as_dict(leave_none)
                 d[key] = value
         return d
 
@@ -277,7 +299,7 @@ class Scheme(abc.Mapping):
         s += ', '.join(
             [
                 f'{field}={repr(getattr(self, field))}'
-                for field in self._fields
+                for field in sorted(self._fields)
             ]
         )
         return s + ')'
